import "tns-core-modules/globals";
import { File, knownFolders } from 'tns-core-modules/file-system';
import * as app from "tns-core-modules/application";

declare var android, AVAudioPlayer;
const context: Worker = self as any;

var apiKey = "Api Key";
var region = "westus2";

context.onmessage = msg => {

    let data = JSON.parse(msg.data);
    console.log(data);

    if (data.task == "playAudio") {
        playAudio(data.content);
    }
    else if (data.task == "record") {
        record();
    }

};//

function playAudio(content) {

    if (app.ios) {

        let speechConfig = SPXSpeechConfiguration.alloc().initWithSubscriptionRegion(apiKey, region);

        speechConfig.setSpeechSynthesisOutputFormat(SPXSpeechSynthesisOutputFormat.Audio16Khz32KBitRateMonoMp3)
        speechConfig.speechSynthesisVoiceName = "en-US-GuyNeural";

        let speechSynthesizer = SPXSpeechSynthesizer.alloc().initWithSpeechConfigurationAudioConfiguration(speechConfig, null);

        let speechResult = speechSynthesizer.speakText(content);

        if (speechResult.reason == SPXResultReason.SynthesizingAudioCompleted) {
            console.log(speechResult.reason);

            let player = AVAudioPlayer.alloc().initWithDataError(speechResult.audioData);
            player.enableRate = true;
            player.prepareToPlay();
            player.rate = 0.9;
            player.play();

            console.log("player.duration: " + ((player.duration * 1000) + 100));

            setTimeout(() => {
                (<any>global).postMessage("done");
            }, ((player.duration * 1000) + 100));


        } else {
            let details = SPXSpeechSynthesisCancellationDetails.alloc().initFromCanceledSynthesisResult(speechResult);
            (<any>global).postMessage(details.errorDetails);
        }
    } else {
        let ResultReason = com.microsoft.cognitiveservices.speech.ResultReason;
        let SpeechConfig = com.microsoft.cognitiveservices.speech.SpeechConfig;
        let SpeechSynthesisCancellationDetails = com.microsoft.cognitiveservices.speech.SpeechSynthesisCancellationDetails;
        let SpeechSynthesizer = com.microsoft.cognitiveservices.speech.SpeechSynthesizer;
        const file = knownFolders.temp().path + "/recordedFile.mp3";

        let speechConfig = SpeechConfig.fromSubscription(apiKey, region);
        speechConfig.setSpeechSynthesisOutputFormat(com.microsoft.cognitiveservices.speech.SpeechSynthesisOutputFormat.Audio16Khz32KBitRateMonoMp3);
        speechConfig.setSpeechSynthesisVoiceName("en-US-GuyNeural");

        let synthesizer = new SpeechSynthesizer(speechConfig); //

        let result = synthesizer.SpeakText(content);

        if (result.getReason() == ResultReason.SynthesizingAudioCompleted) {
            console.log("Speech synthesis succeeded.");

            let audioFile = File.fromPath(file);
            audioFile.write(result.getAudioData()).then((res) => {

                let mediaPlayer = new android.media.MediaPlayer();
                mediaPlayer.setDataSource(file);
                mediaPlayer.prepare();
                mediaPlayer.start();
                //console.log(mediaPlayer.getDuration());

                mediaPlayer.setOnCompletionListener(new android.media.MediaPlayer.OnCompletionListener({
                    onCompletion() {
                        console.log("onCompletion");
                        (<any>global).postMessage("done");
                    }
                }));
            });


        } else {
            let details = SpeechSynthesisCancellationDetails.fromResult(result);
            let msg = details.getErrorDetails();
            if (!msg) {
                msg = result.toString();
            }
            (<any>global).postMessage(msg);
        }

        speechConfig.close();
    }
}

function record() {
    if (app.ios) {

        let speechConfig = SPXSpeechConfiguration.alloc().initWithSubscriptionRegion(apiKey, region);
        speechConfig.outputFormat = SPXOutputFormat.Detailed;

        let speechRecognizer = SPXSpeechRecognizer.alloc().init(speechConfig);

        let speechResult = speechRecognizer.recognizeOnce();
        if (speechResult.reason == SPXResultReason.RecognizedSpeech) {

            let properties = speechResult.properties;
            let speechJson = properties.getPropertyById(SPXPropertyId.SpeechServiceResponseJsonResult);

            console.log(speechJson);
            (<any>global).postMessage(JSON.stringify(speechJson));


        } else {
            let details = SPXCancellationDetails.alloc().initFromCanceledRecognitionResult(speechResult);
            console.log(details.errorDetails);

            let sendInfo: any = {};
            sendInfo.gotResult = false;

            if (details.errorDetails) {
                sendInfo.msg = details.errorDetails;
            } else {
                sendInfo.msg = "NoMatch"
            }

            (<any>global).postMessage(JSON.stringify(sendInfo));
            //close();
        }
    } else {

        let ResultReason = com.microsoft.cognitiveservices.speech.ResultReason;
        let SpeechConfig = com.microsoft.cognitiveservices.speech.SpeechConfig;
        let SpeechRecognizer = com.microsoft.cognitiveservices.speech.SpeechRecognizer;
        let PhraseListGrammar = com.microsoft.cognitiveservices.speech.PhraseListGrammar;
        let CancellationDetails = com.microsoft.cognitiveservices.speech.CancellationDetails;

        let config = SpeechConfig.fromSubscription(apiKey, region);
        config.setOutputFormat(com.microsoft.cognitiveservices.speech.OutputFormat.Detailed);

        let speechRecognizer = new SpeechRecognizer(config);

        //let phraseListGrammar = PhraseListGrammar.fromRecognizer(speechRecognizer);
        //phraseListGrammar.addPhrase("Hello, I am testing");

        let task = speechRecognizer.recognizeOnceAsync();

        let result = task.get();

        if (result.getReason() == ResultReason.RecognizedSpeech) {

            let properties = result.getProperties();
            let PropertyId = com.microsoft.cognitiveservices.speech.PropertyId;
            let speechJson = properties.getProperty(PropertyId.SpeechServiceResponse_JsonResult);
            console.dir(speechJson)
        }
        else {
            let details = CancellationDetails.fromResult(result);
            console.log(details.getErrorDetails());
            let sendInfo: any = {};
            sendInfo.gotResult = false;
            sendInfo.msg = details.getErrorDetails();
            if (!sendInfo.msg) {
                sendInfo.msg = result.toString();
            }

            (<any>global).postMessage(JSON.stringify(sendInfo));
            //close();
        }
        config.close();
    }
}
